@@ -95,7 +95,7 @@ def register_rebalance_class(name=None, register_in_agent=True):
 
     def decorator(cls):
         if os.environ.get("IGNORE_MS_AGENT", "false").lower() == "true":
-            logger.info("Ignoring MS agent registration")
+            logger.warning("Ignoring MS agent registration")
             return cls
 
         return insert_in_registry(REBALANCE_CLASS_REGISTRY, cls, register_in_agent, name)
