@@ -1,31 +1,21 @@
-import copy
-import enum
 import inspect
 import json
-import logging
-from typing import Optional, Union, get_origin, get_args
+from datetime import datetime
+from enum import Enum
+from typing import Any, Union, get_args, get_origin, get_type_hints
 
-from pydantic.fields import PydanticUndefined, FieldInfo
-
-import pandas as pd
-from mainsequence.client import CONSTANTS, Asset
-from mainsequence.tdag.data_nodes import  DataNode
+import docstring_parser
 import numpy as np
-from tqdm import tqdm
+import pandas as pd
+from joblib import Parallel, delayed
 from numpy.linalg import LinAlgError
 from numpy.typing import NDArray
-from joblib import delayed, Parallel
-from typing import Dict, Any, List, Tuple, Union, Optional
-from datetime import datetime
-import docstring_parser
-from typing import Any, Dict, List, Union, get_type_hints
-from enum import Enum
 from pydantic import BaseModel
-import yaml
+from tqdm import tqdm
+
+from mainsequence.client import Asset
 from mainsequence.logconf import logger
-import inspect
-from typing import Any, Dict, List, Optional, Type, Union
-from pydantic import BaseModel, Field, create_model
+
 
 def get_vfb_logger():
     global logger
@@ -34,6 +24,7 @@ def get_vfb_logger():
     logger.bind(sub_application="virtualfundbuilder")
     return logger
 
+
 logger = get_vfb_logger()
 
 # Symbol mapping for CoinGecko API
@@ -48,17 +39,22 @@ GECKO_SYMBOL_MAPPING = {
     "SOL": "solana",
     "ATOM": "cosmos",
     "BTC": "bitcoin",
-    "ETH": "ethereum"
+    "ETH": "ethereum",
 }
 
 # Small time delta for precision operations
 TIMEDELTA = pd.Timedelta("5ms")
 
+
 def runs_in_main_process() -> bool:
     import multiprocessing
+
     return multiprocessing.current_process().name == "MainProcess"
 
-def reindex_df(df: pd.DataFrame, start_time: datetime, end_time: datetime, freq: str) -> pd.DataFrame:
+
+def reindex_df(
+    df: pd.DataFrame, start_time: datetime, end_time: datetime, freq: str
+) -> pd.DataFrame:
     """
     Aligns two DataFrames on a new index based on a specified frequency, filling missing entries with the last known values.
 
@@ -74,6 +70,7 @@ def reindex_df(df: pd.DataFrame, start_time: datetime, end_time: datetime, freq:
     new_index = pd.date_range(start=start_time, end=end_time, freq=freq)
     return df.reindex(new_index).ffill()
 
+
 def convert_to_binance_frequency(freq: str) -> str:
     """
     Converts a generic frequency format to a format compatible with Binance API requirements.
@@ -84,22 +81,21 @@ def convert_to_binance_frequency(freq: str) -> str:
     Returns:
         str: A frequency string adapted for Binance API (e.g., '1m', '1h').
     """
-    frequency_mappings = {'min': 'm', 'h': 'h', 'd': 'd', 'w': 'w'}  # TODO extend
+    frequency_mappings = {"min": "m", "h": "h", "d": "d", "w": "w"}  # TODO extend
     for unit, binance_unit in frequency_mappings.items():
         if freq.endswith(unit):
-            return freq[:-len(unit)] + binance_unit
+            return freq[: -len(unit)] + binance_unit
     raise NotImplementedError(f"Frequency of {freq} not supported")
 
 
-
 def get_last_query_times_per_asset(
-        latest_value: datetime,
-        data_node_storage: dict,
-        asset_list: List[Asset],
-        max_lookback_time: datetime,
-        current_time: datetime,
-        query_frequency: str
-) -> Dict[str, Optional[float]]:
+    latest_value: datetime,
+    data_node_storage: dict,
+    asset_list: list[Asset],
+    max_lookback_time: datetime,
+    current_time: datetime,
+    query_frequency: str,
+) -> dict[str, float | None]:
     """
     Determines the last query times for each asset based on metadata, a specified lookback limit, and a query frequency.
 
@@ -115,7 +111,9 @@ def get_last_query_times_per_asset(
         Dict[str, Optional[float]]: A dictionary mapping asset IDs to their respective last query times expressed in UNIX timestamp.
     """
     if latest_value:
-        last_query_times_per_asset = data_node_storage["sourcetableconfiguration"]["multi_index_stats"]["max_per_asset_symbol"]
+        last_query_times_per_asset = data_node_storage["sourcetableconfiguration"][
+            "multi_index_stats"
+        ]["max_per_asset_symbol"]
     else:
         last_query_times_per_asset = {}
 
@@ -127,14 +125,24 @@ def get_last_query_times_per_asset(
             asset_start_time = max_lookback_time
 
         if asset_start_time >= (current_time - pd.Timedelta(query_frequency)):
-            logger.info(f"no new data for asset {asset.name} from {asset_start_time} to {current_time}")
+            logger.info(
+                f"no new data for asset {asset.name} from {asset_start_time} to {current_time}"
+            )
             last_query_times_per_asset[asset_id] = None
         else:
             last_query_times_per_asset[asset_id] = (asset_start_time + TIMEDELTA).timestamp()
 
     return last_query_times_per_asset
 
-def do_single_regression(xx: NDArray, XTX_inv_list: list, rolling_window: int, col_name: str, tmp_y: NDArray, XTX_inv_diag: list) -> pd.DataFrame:
+
+def do_single_regression(
+    xx: NDArray,
+    XTX_inv_list: list,
+    rolling_window: int,
+    col_name: str,
+    tmp_y: NDArray,
+    XTX_inv_diag: list,
+) -> pd.DataFrame:
     """
     Performs a single regression analysis on a sliding window of data points for a specific column.
 
@@ -158,22 +166,31 @@ def do_single_regression(xx: NDArray, XTX_inv_list: list, rolling_window: int, c
         xxx = xx[i].reshape(rolling_window, xx[i].shape[-1])
         tmpy_ = tmp_y[i]
         x_mult = XTX_inv_list[i] @ (xxx.T)
-        coefs = (x_mult @ tmpy_.T)
+        coefs = x_mult @ tmpy_.T
         y_estimates = (xxx @ coefs.reshape(-1, 1)).ravel()
         residuals = tmpy_ - y_estimates
         SSR = np.sum((y_estimates - precompute_y_["mean_y"][i]) ** 2)
         rsquared = SSR / precompute_y_["SST"][i]
-        residuals_var = np.sum(residuals ** 2) / (rolling_window - coefs.shape[0] + 1)
+        residuals_var = np.sum(residuals**2) / (rolling_window - coefs.shape[0] + 1)
         standard_errors = np.sqrt(XTX_inv_diag[i] * residuals_var)
         ts = coefs / standard_errors
-        results.append(dict(beta=coefs[0], intercept=coefs[1],
-                            rsquared=rsquared, t_intercept=ts[1], t_beta=ts[0]
-                            ))
+        results.append(
+            dict(
+                beta=coefs[0],
+                intercept=coefs[1],
+                rsquared=rsquared,
+                t_intercept=ts[1],
+                t_beta=ts[0],
+            )
+        )
     results = pd.concat([pd.DataFrame(results)], keys=[col_name], axis=1)
 
     return results
 
-def build_rolling_regression_from_df(x: NDArray, y: NDArray, rolling_window: int, column_names: list, threads: int=5) -> pd.DataFrame:
+
+def build_rolling_regression_from_df(
+    x: NDArray, y: NDArray, rolling_window: int, column_names: list, threads: int = 5
+) -> pd.DataFrame:
     """
     Builds rolling regressions for multiple variables in parallel using a specified rolling window.
 
@@ -190,7 +207,10 @@ def build_rolling_regression_from_df(x: NDArray, y: NDArray, rolling_window: int
     XX = np.concatenate([x.reshape(-1, 1), np.ones((x.shape[0], 1))], axis=1)
     xx = np.lib.stride_tricks.sliding_window_view(XX, (rolling_window, XX.shape[1]))
 
-    XTX_inv_list, XTX_inv_diag = [], []  # pre multiplication of x before y and diagonal for standard errros
+    XTX_inv_list, XTX_inv_diag = (
+        [],
+        [],
+    )  # pre multiplication of x before y and diagonal for standard errros
 
     # precompute for x
     for i in tqdm(range(xx.shape[0]), desc="building x precomputes"):
@@ -199,18 +219,27 @@ def build_rolling_regression_from_df(x: NDArray, y: NDArray, rolling_window: int
             XTX_inv = np.linalg.inv(xxx.T @ xxx)
             XTX_inv_list.append(XTX_inv)
             XTX_inv_diag.append(np.diag(XTX_inv))
-        except LinAlgError as le:
+        except LinAlgError:
             XTX_inv_list.append(XTX_inv_list[-1] * np.nan)
             XTX_inv_diag.append(XTX_inv_diag[-1] * np.nan)
 
-    y_views = {i: np.lib.stride_tricks.sliding_window_view(y[:, i], (rolling_window,)) for i in range(y.shape[1])}
+    y_views = {
+        i: np.lib.stride_tricks.sliding_window_view(y[:, i], (rolling_window,))
+        for i in range(y.shape[1])
+    }
 
     work_details = dict(n_jobs=threads, prefer="threads")
     reg_results = Parallel(**work_details)(
-        delayed(do_single_regression)(xx=xx, tmp_y=tmp_y, XTX_inv_list=XTX_inv_list,
-                                      rolling_window=rolling_window,
-                                      XTX_inv_diag=XTX_inv_diag, col_name=column_names[y_col]
-                                      ) for y_col, tmp_y in y_views.items())
+        delayed(do_single_regression)(
+            xx=xx,
+            tmp_y=tmp_y,
+            XTX_inv_list=XTX_inv_list,
+            rolling_window=rolling_window,
+            XTX_inv_diag=XTX_inv_diag,
+            col_name=column_names[y_col],
+        )
+        for y_col, tmp_y in y_views.items()
+    )
 
     reg_results = pd.concat(reg_results, axis=1)
     reg_results.columns = reg_results.columns.swaplevel()
@@ -224,18 +253,20 @@ def parse_google_docstring(docstring):
         "args_descriptions": {param.arg_name: param.description for param in parsed.params},
         "returns": parsed.returns.description if parsed.returns else None,
         "example": "\n".join([param.description for param in parsed.examples]),
-        "raises": {exc.type_name: exc.description for exc in parsed.raises}
+        "raises": {exc.type_name: exc.description for exc in parsed.raises},
     }
 
+
 def extract_code(output_string):
     import re
+
     # Use regex to find content between triple backticks
-    match = re.search(r'```[^\n]*\n(.*?)```', output_string, re.DOTALL)
+    match = re.search(r"```[^\n]*\n(.*?)```", output_string, re.DOTALL)
     if match:
         code = match.group(1)
         return code
     else:
-        return ''
+        return ""
 
 
 def _convert_unknown_to_string(obj):
@@ -245,15 +276,17 @@ def _convert_unknown_to_string(obj):
     except Exception:
         raise TypeError(f"Object of type {type(obj).__name__} is not JSON serializable")
 
+
 def is_jupyter_environment():
     try:
         from IPython import get_ipython
+
         return "ipykernel" in str(get_ipython())
     except ImportError:
         return False
 
 
-def type_to_json_schema(py_type: Type, definitions: Dict[str, Any]) -> Dict[str, Any]:
+def type_to_json_schema(py_type: type, definitions: dict[str, Any]) -> dict[str, Any]:
     """
     Recursively converts a Python type annotation to a JSON schema dictionary.
     Handles Pydantic models, Enums, Lists, Unions, and basic types.
@@ -282,10 +315,10 @@ def type_to_json_schema(py_type: Type, definitions: Dict[str, Any]) -> Dict[str,
 
     if origin is Union:
         return {"anyOf": [type_to_json_schema(arg, definitions) for arg in args]}
-    if origin in (list, List):
+    if origin in (list, list):
         item_schema = type_to_json_schema(args[0], definitions) if args else {}
         return {"type": "array", "items": item_schema}
-    if origin in (dict, Dict):
+    if origin in (dict, dict):
         value_schema = type_to_json_schema(args[1], definitions) if len(args) > 1 else {}
         return {"type": "object", "additionalProperties": value_schema}
 
@@ -314,10 +347,13 @@ def type_to_json_schema(py_type: Type, definitions: Dict[str, Any]) -> Dict[str,
         return {}  # Any type, no constraint
 
     # Fallback for unknown types
-    return {"type": "string", "description": f"Unrecognized type: {getattr(py_type, '__name__', str(py_type))}"}
+    return {
+        "type": "string",
+        "description": f"Unrecognized type: {getattr(py_type, '__name__', str(py_type))}",
+    }
 
 
-def create_schema_from_signature(func: callable) -> Dict[str, Any]:
+def create_schema_from_signature(func: callable) -> dict[str, Any]:
     """
     Parses a function's signature (like __init__) and creates a JSON schema.
 
@@ -330,7 +366,7 @@ def create_schema_from_signature(func: callable) -> Dict[str, Any]:
     try:
         signature = inspect.signature(func)
         type_hints = get_type_hints(func)
-    except (TypeError, NameError): # Handles cases where hints can't be resolved
+    except (TypeError, NameError):  # Handles cases where hints can't be resolved
         return {}
 
     parsed_doc = docstring_parser.parse(func.__doc__ or "")
@@ -341,15 +377,15 @@ def create_schema_from_signature(func: callable) -> Dict[str, Any]:
     definitions = {}  # For nested models
 
     for name, param in signature.parameters.items():
-        if name in ('self', 'cls', 'args', 'kwargs'):
+        if name in ("self", "cls", "args", "kwargs"):
             continue
 
         param_type = type_hints.get(name, Any)
         prop_schema = type_to_json_schema(param_type, definitions)
-        prop_schema['title'] = name.replace('_', ' ').title()
+        prop_schema["title"] = name.replace("_", " ").title()
 
         if name in arg_descriptions:
-            prop_schema['description'] = arg_descriptions[name]
+            prop_schema["description"] = arg_descriptions[name]
 
         if param.default is inspect.Parameter.empty:
             required.append(name)
@@ -358,18 +394,18 @@ def create_schema_from_signature(func: callable) -> Dict[str, Any]:
             try:
                 # Ensure default is JSON serializable
                 json.dumps(default_value)
-                prop_schema['default'] = default_value
+                prop_schema["default"] = default_value
             except TypeError:
-                 if isinstance(default_value, Enum):
-                     prop_schema['default'] = default_value.value
-                 else:
-                     # Fallback for non-serializable defaults
-                     prop_schema['default'] = str(default_value)
+                if isinstance(default_value, Enum):
+                    prop_schema["default"] = default_value.value
+                else:
+                    # Fallback for non-serializable defaults
+                    prop_schema["default"] = str(default_value)
 
         properties[name] = prop_schema
 
     schema = {
-        "title": getattr(func, '__name__', 'Schema'),
+        "title": getattr(func, "__name__", "Schema"),
         "type": "object",
         "properties": properties,
     }
